package org.bench4q.recorder.httpcapture.generator;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.net.HttpURLConnection;
import java.nio.charset.Charset;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;

import org.apache.commons.httpclient.ChunkedInputStream;
import org.apache.commons.io.output.ByteArrayOutputStream;
import org.apache.log4j.Logger;
import org.bench4q.recorder.httpcapture.ResponseModel;

public class ResponseParser {
	private Logger logger = Logger.getLogger(ResponseParser.class);
	private byte[] response;
	private ResponseHeader responseHeader;
	private String responseBody;

	private byte[] getResponse() {
		return response;
	}

	private void setResponse(byte[] response) {
		this.response = response;
	}

	public ResponseHeader getResponseHeader() {
		return responseHeader;
	}

	private void setResponseHeader(ResponseHeader responseHeader) {
		this.responseHeader = responseHeader;
	}

	public String getResponseBody() {
		return responseBody;
	}

	private void setResponseBody(String responseBody) {
		this.responseBody = responseBody;
	}

	public ResponseParser(ResponseModel response) {
		this.setResponse(response.getResponse());
		this.setResponseHeader(parseResponseHeader(response.getConn()));
		if (this.getResponseHeader().isValidResponseHeader()
				&& this.getResponseHeader().isHtmlContentType()) {
//			this.parseResponseBody();
			this.setResponseBody(new String(response.getResponse(),Charset.forName("utf-8")));
		
		}
	}

	private ResponseHeader parseResponseHeader(
			HttpURLConnection httpURLConnection) {
		ResponseHeader result = new ResponseHeader();
		if(httpURLConnection == null) return result;
		result.setContentEncoding(httpURLConnection.getContentEncoding());
		result.setContentLength(httpURLConnection.getContentLength());
		try {
			result.setRespCode(httpURLConnection.getResponseCode());
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		result.setTransferEncoding(parseTransferEncoding(httpURLConnection));
		result.setCharset(parseCharset(httpURLConnection));
		result.setContentType(parseContentType(httpURLConnection));
		return result;
	}

	private String preprocess(String respString) {
		return respString.toLowerCase();
	}

	private String parseContentLength(Map<String, List<String>> map) {
		// String respStr = preprocess(new String(this.getResponse()));
		// int pos = respStr.indexOf("content-length:");
		// if (pos > -1) {
		// pos += 15;
		// int end = respStr.indexOf("\r\n", pos);
		// if (end == -1) {
		// return "-1";
		// }
		// return respStr.substring(pos, end).trim();
		// }
		// return "-1";
		List<String> values = map.get("Content-Length:");
		if (values != null) {
			return values.get(0);
		}
		return "-1";
	}

	private String parseContentEncoding(Map<String, List<String>> map) {
		// String respStr = preprocess(new String(this.getResponse()));
		// int pos = respStr.indexOf("content-encoding:");
		// if (pos > -1) {
		// pos += 18;
		// int end = respStr.indexOf("\r\n", pos);
		// return respStr.substring(pos, end);
		// }
		String ret = "";
		List<String> values = map.get("Content-Encoding:");
		if (values != null) {
			return values.get(0);
		}
		return null;
	}

	private String parseCharset(HttpURLConnection httpURLConnection) {
		// String response = preprocess(new String(this.getResponse()));
		// String ret = null;
		// int pos = response.indexOf("content-type:");
		// if (pos > -1) {
		// pos += 14;
		// int end = response.indexOf("\r\n", pos);
		// int middle = response.indexOf(";", pos);
		// if (middle > -1 && middle < end) {
		// ret = response.substring(middle + 1, end);
		// }
		// }
		// if (ret != null) {
		// int begin = ret.indexOf("charset=");
		// ret = ret.substring(begin + 8);
		// }
		String ret = null;
		String value = httpURLConnection.getHeaderField("Content-Type");
		int pos;
		if (value != null && (pos = value.indexOf("charset=")) != -1) {
			ret = value.substring(pos + 8);
		}

		return ret;
	}

	private String parseContentType(HttpURLConnection httpURLConnection) {
		// String response = preprocess(new String(this.getResponse()));
		// String contentType = null;
		// int pos = response.indexOf("content-type:");
		// if (pos > -1) {
		// pos += 14;
		// int end = response.indexOf("\r\n", pos);
		// int end2 = response.indexOf(";", pos);
		// if ((end2 > -1) && (end2 < end))
		// end = end2;
		// if (end > -1)
		// contentType = response.substring(pos, end).trim();
		//
		// logger.debug("  Content-Type: " + contentType);
		// } else {
		// logger.debug("  No content-type header!  First few lines:");
		// StringTokenizer st = new StringTokenizer(response, "\n");
		// int i = 0;
		// while ((st.hasMoreTokens()) && (i < 5)) {
		// logger.debug(st.nextToken());
		// ++i;
		// }
		// }
		// return contentType;
		String ret = null;
		String value = httpURLConnection.getHeaderField("Content-Type");
		int pos;
		if (value != null) {
			if ((pos = value.indexOf(";")) != -1) {
				ret = value.substring(0, pos);
			} else {

				ret = value;
			}
		}

		return ret;
	}

	// private String parseResponseCode() {
	// String response = preprocess(new String(this.getResponse()));
	// String respCode = null;
	// int pos = response.indexOf(" ");
	// if (pos != -1) {
	// int end = response.indexOf(" ", pos + 1);
	// int end2 = response.indexOf("\n", pos + 1);
	// if ((end2 != -1) && (end2 < end))
	// end = end2;
	// if (end != -1)
	// respCode = response.substring(pos + 1, end).trim();
	// }
	// logger.debug("HTTP response code: " + respCode);
	// return respCode;
	// }

	private String parseTransferEncoding(HttpURLConnection httpURLConnection) {
		// String response = preprocess(new String(this.getResponse()));
		// int pos = response.indexOf("transfer-encoding:");
		// if (pos > -1) {
		// pos += 19;
		// int end = response.indexOf("\r\n", pos);
		// return response.substring(pos, end);
		// }
		// return null;
		return httpURLConnection.getHeaderField("Transfer-Encoding:");
	}

	private byte[] buildResponseBody() {
		String response = preprocess(new String(this.getResponse()));
		int pos = response.indexOf("\r\n\r\n");
		pos += 4;
		String testRes = response.substring(pos);
		System.out.println(testRes);
		ByteArrayInputStream in = new ByteArrayInputStream(this.getResponse(),
				pos, this.getResponse().length - pos);
		ChunkedInputStream chunkedIS = null;
		ByteArrayOutputStream out = new ByteArrayOutputStream();
		try {

			chunkedIS = new ChunkedInputStream(in);
			int tempByte;
			while ((tempByte = chunkedIS.read()) != -1) {
				out.write(tempByte);
			}

			return out.toByteArray();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} finally {
			try {
				if (chunkedIS != null) {
					chunkedIS.close();
				}
				if (out != null) {
					out.close();
				}
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		return null;
	}

	private void parseResponseBody() {
		ContentDecoder contentDecoder = ContentDecoder.createDecoder(this
				.getResponseHeader().getContentEncoding());
		byte[] contentBodyAfterDecoded = null;
		if (this.getResponseHeader().getTransferEncoding() != null
				&& this.getResponseHeader().getTransferEncoding()
						.equals("chunked")) {

			contentBodyAfterDecoded = contentDecoder
					.decodeContent(new ByteArrayInputStream(buildResponseBody()));
		} else {
			contentBodyAfterDecoded = contentDecoder
					.decodeContent(new ByteArrayInputStream(this.getResponse(),
							this.getResponse().length
									- this.getResponseHeader()
											.getContentLength(), this
									.getResponseHeader().getContentLength()));
		}
		if (contentBodyAfterDecoded == null) {
			return;
		}
		Charset charset = null;
		try {
			charset = Charset.forName(this.getResponseHeader().getCharset());
		} catch (Exception e) {
			charset = Charset.forName("utf-8");
			logger.error(e, e);
		}
		this.setResponseBody(new String(contentBodyAfterDecoded, charset));
	}
}
